unit InfoDialog;

interface

uses Windows, Messages, ShellAPI, NLib, NForm, NkvImage;

type
  TInfoDlg = class(TModelessDlg)
    Path: string;
    PathInArchive: string;
    Image: TNkVImage;
    Icon: HICON;
    CTime, ATime, MTime: TFileTime;
    procedure UpdateInfo;
    procedure WMDrawItem(var Message: TWMDrawItem); message WM_DRAWITEM;
    procedure WMInitDialog(var Message: TWMInitDialog); message WM_INITDIALOG;
    procedure WMCommand(var Message: TWMCommand); message WM_COMMAND; 
  end;

implementation

{$I dialogid.inc}

function GetIcon(const Path: string): HICON;
var
  id: Word;
begin
  id := 1; // if id=1, It is not thumbnail icon but normal file icon
  Result := ExtractAssociatedIcon(HInstance, PChar(Path), id);
end;

function BytesFormat(Value: Integer): string;
var
  s: string;
begin
  Result := UnitFormat(Value);
  s := Copy(Result, 1, Length(Result)-1);
  case Result[Length(Result)] of
    'G': Result := s + ' GB';
    'M': Result := s + ' MB';
    'K': Result := s + ' KB';
  else
    Result := Result + ' bytes';
  end;
end;

function NumFormat(Value: Integer): string;
var
  nf: TNumberFmt;
  buf: array[0..255] of Char;
begin
  nf.NumDigits := 0;
  nf.LeadingZero := 0;
  nf.Grouping := 3;
  nf.lpDecimalSep := '.';
  nf.lpThousandSep := ',';
  nf.NegativeOrder := 1;
  GetNumberFormat(0, 0, PChar(IStr(Value)), @nf, buf, SizeOf(buf));
  Result := buf;
end;

function DateTimeStr(const FileTime: TFileTime): string;
var
  ft: TFileTime;
  st: TSystemTime;
begin
  if (FileTime.dwLowDateTime = 0) and (FileTime.dwHighDateTime = 0) then
    Result := ''
  else begin
    FileTimeToLocalFileTime(FileTime, ft);
    FileTimeToSystemTime(ft, st);
    with st do
      Result := Fmt('%s-%02s-%02s %02s:%02s:%02s',
          [wYear, wMonth, wDay, wHour, wMinute, wSecond]);
  end;
end;

procedure TInfoDlg.UpdateInfo;
begin
  if Path = '' then
  begin
    Icon := 0;
    Items[IDC_FILENAME].SetText('<STDIN>')
  end else
  begin
    Icon := GetIcon(Path);
    if PathInArchive = '' then
      Items[IDC_FILENAME].SetText(BaseName(Path))
    else
      Items[IDC_FILENAME].SetText(PathInArchive)
  end;
  if PathInArchive = '' then
    Items[IDC_LOCATION].SetText(DirName(Path))
  else
    Items[IDC_LOCATION].SetText(Path);
  Items[IDC_SIZE].SetText(Fmt('%sx%s', [Image.Width, Image.Height]));
  Items[IDC_BPP].SetText(Fmt('%s bits/pixel', [Image.Bpp]));
  Items[IDC_FORMAT].SetText(Image.FormatName);
  Items[IDC_FILESIZE].SetText(Fmt('%s (%s bytes)', [BytesFormat(Image.OrgSize),
    NumFormat(Image.OrgSize)]));
  Items[IDC_CREATED].SetText(DateTimeStr(CTime));
  Items[IDC_MODIFIED].SetText(DateTimeStr(MTime));
  Items[IDC_ACCESSED].SetText(DateTimeStr(ATime));
  Items[IDC_MEMORY].SetText(Fmt('%s (%s bytes)', [BytesFormat(Image.BitsSize),
    NumFormat(Image.BitsSize)]));
  Items[IDC_PLUGIN].SetText(Image.FilterName);
  Items[IDC_PLUGINFILE].SetText(Image.FilterFilename);
  Items[IDC_TEXTINFOLABEL].Enable(Image.Comment <> '');
  Items[IDC_TEXTINFO].SetText(Image.Comment);
  InvalidateRect(Items[IDC_ICON].Handle, nil, False);
end;

procedure TInfoDlg.WMDrawItem(var Message: TWMDrawItem);
begin
  with Message do
    if Ctl = IDC_ICON  then
      with DrawItemStruct^ do
      begin
        FillRect(hDC, rcItem, GetSysColorBrush(COLOR_3DFACE));
        DrawIcon(hDC, 0, 0, Icon);
        Result := 1;
      end;
end;

procedure TInfoDlg.WMInitDialog(var Message: TWMInitDialog);
begin
  UpdateInfo;
  Message.Result := 0;
end;

procedure TInfoDlg.WMCommand(var Message: TWMCommand);
begin
  with Message do
    case ItemID of
      IDOK, IDCANCEL:
        begin
          Result := 1;
          Close;
        end;
    end;
end;

end.

